@@ -7,21 +7,21 @@
 :License: MIT
 """
 
+from .config import Config
 from .data_model import OutputMedium
 from .exec_core import SimulatorValidator
 from .results.data_model import TestCaseResult, TestCaseResultType, TestResultsReport  # noqa: F401
 from .results.io import write_test_results
+from biosimulators_utils.biosimulations.utils import validate_biosimulations_api_response
 from biosimulators_utils.gh_action.data_model import Comment, GitHubActionCaughtError  # noqa: F401
 from biosimulators_utils.gh_action.core import GitHubAction, GitHubActionErrorHandling
 from biosimulators_utils.image import get_docker_image
 from biosimulators_utils.simulator_registry.data_model import SimulatorSubmission, IssueLabel  # noqa: F401
 from biosimulators_utils.simulator_registry.process_submission import get_simulator_submission_from_gh_issue_body
 from biosimulators_utils.simulator_registry.query import get_simulator_version_specs
-
 from natsort import natsort_keygen
 import biosimulators_utils.image
 import biosimulators_utils.simulator.io
-import os
 import requests
 
 
@@ -59,17 +59,10 @@ class ValidateCommitSimulatorGitHubAction(GitHubAction):
     Attributes:
         issue_number (:obj:`str`): number of GitHub issue which triggered the action
     """
-    BIOSIMULATORS_AUTH_ENDPOINT = 'https://auth.biosimulations.org/oauth/token'
-    BIOSIMULATORS_AUDIENCE = os.getenv('BIOSIMULATORS_AUDIENCE', 'api.biosimulators.org')
-    BIOSIMULATORS_API_ENDPOINT = os.getenv('BIOSIMULATORS_API_ENDPOINT', 'https://api.biosimulators.org/')
-    DOCKER_REGISTRY_URL = 'ghcr.io'
-    DOCKER_REGISTRY_IMAGE_URL_PATTERN = 'ghcr.io/biosimulators/{}:{}'
-    DEFAULT_SPECIFICATIONS_VERSION = '1.0.0'
-    DEFAULT_IMAGE_VERSION = '1.0.0'
-    CURATOR_GH_IDS = ['jonrkarr']
 
     def __init__(self):
         super(ValidateCommitSimulatorGitHubAction, self).__init__()
+        self.config = Config()
         self.issue_number = self.get_issue_number()
 
     @GitHubActionErrorHandling.catch_errors(uncaught_exception_msg_func=get_uncaught_exception_msg,
@@ -126,7 +119,7 @@ class ValidateCommitSimulatorGitHubAction(GitHubAction):
                 self.close_issue(self.issue_number)
             else:
                 assigned = set([assignee['login'] for assignee in issue_props['assignees']])
-                new_assignees = set(self.CURATOR_GH_IDS).difference(assigned)
+                new_assignees = set(self.config.biosimulators_curator_gh_ids).difference(assigned)
                 if new_assignees:
                     self.assign_issue(self.issue_number,  list(new_assignees))
                 self.add_comment_to_issue(
@@ -208,9 +201,7 @@ class ValidateCommitSimulatorGitHubAction(GitHubAction):
             :obj:`list` of :obj:`TestCaseResults`: results of test cases
         """
         docker_client = biosimulators_utils.image.login_to_docker_registry(
-            'docker.io',
-            os.getenv('DOCKER_HUB_USERNAME'),
-            os.getenv('DOCKER_HUB_TOKEN'))
+            'docker.io', self.config.docker_hub_username, self.config.docker_hub_token)
 
         # validate that container (Docker image) exists
         image_url = specifications['image']['url']
@@ -222,7 +213,8 @@ class ValidateCommitSimulatorGitHubAction(GitHubAction):
         # validate that image is consistent with the BioSimulators standards
         validator = SimulatorValidator(specifications, output_medium=OutputMedium.gh_issue)
         case_results = validator.run()
-        write_test_results(case_results, '.biosimulators-test-suite-results.json')
+        write_test_results(case_results, '.biosimulators-test-suite-results.json',
+                           gh_issue=int(self.issue_number), gh_action_run=int(self.get_gh_action_run_id()))
         summary, failure_details, warning_details = validator.summarize_results(case_results)
         msg = '## Summary of tests\n\n{}\n\n'.format(summary)
         if failure_details:
@@ -293,10 +285,14 @@ class ValidateCommitSimulatorGitHubAction(GitHubAction):
             existing_version_specifications (:obj:`list` of :obj:`dict`): specifications of other versions of simulation tool
             test_results (:obj:`list` of :obj:`TestCaseResults`): results of test cases
         """
-        # copy image to BioSimulators namespace of Docker registry (GitHub Container Registry)
+        # commit image
         if submission.validate_image:
+            # copy image to BioSimulators namespace of Docker registry (GitHub Container Registry)
             self.push_image(specifications, existing_version_specifications)
 
+            # instruct runBioSimulations to generate a Singularity image for the Docker image
+            self.trigger_conversion_of_docker_image_to_singularity(specifications)
+
         # commit submission to BioSimulators database
         if 'biosimulators' not in specifications:
             specifications['biosimulators'] = {}
@@ -323,18 +319,16 @@ class ValidateCommitSimulatorGitHubAction(GitHubAction):
         # get image, pulling if necessary
         original_image_url = specifications['image']['url']
         docker_client = biosimulators_utils.image.login_to_docker_registry(
-            'docker.io',
-            os.getenv('DOCKER_HUB_USERNAME'),
-            os.getenv('DOCKER_HUB_TOKEN'))
+            'docker.io', self.config.docker_hub_username, self.config.docker_hub_token)
         image = get_docker_image(docker_client, original_image_url, pull=True)
 
         # push image to BioSimulators namespace of Docker registry
         biosimulators_utils.image.login_to_docker_registry(
-            self.DOCKER_REGISTRY_URL,
-            os.getenv('DOCKER_REGISTRY_USERNAME'),
-            os.getenv('DOCKER_REGISTRY_TOKEN'))
+            self.config.biosimulators_docker_registry_url,
+            self.config.biosimulators_docker_registry_username,
+            self.config.biosimulators_docker_registry_token)
 
-        copy_image_url = self.DOCKER_REGISTRY_IMAGE_URL_PATTERN \
+        copy_image_url = self.config.biosimulators_docker_image_url_pattern \
             .format(specifications['id'], specifications['version']) \
             .lower()
         biosimulators_utils.image.tag_and_push_docker_image(docker_client, image, copy_image_url)
@@ -343,7 +337,7 @@ class ValidateCommitSimulatorGitHubAction(GitHubAction):
         is_latest = self.is_submission_latest_version_of_simulator(specifications, existing_version_specifications)
 
         if is_latest:
-            latest_copy_image_url = self.DOCKER_REGISTRY_IMAGE_URL_PATTERN \
+            latest_copy_image_url = self.config.biosimulators_docker_image_url_pattern \
                 .format(specifications['id'], 'latest') \
                 .lower()
             biosimulators_utils.image.tag_and_push_docker_image(docker_client, image, latest_copy_image_url)
@@ -370,6 +364,24 @@ class ValidateCommitSimulatorGitHubAction(GitHubAction):
                 return False
         return True
 
+    def trigger_conversion_of_docker_image_to_singularity(self, specifications):
+        """ Post the simulation to the BioSimulators database
+
+        Args:
+            specifications (:obj:`dict`): specifications of a simulation tool
+        """
+        auth_headers = self.get_auth_headers_for_biosimulations_api(
+            self.config.runbiosimulations_auth_endpoint, self.config.runbiosimulations_audience,
+            self.config.runbiosimulations_api_client_id, self.config.runbiosimulations_api_client_secret)
+
+        response = requests.post(self.config.runbiosimulations_api_endpoint + 'images/refresh',
+                                 headers=auth_headers,
+                                 json={
+                                     'simulator': specifications['id'],
+                                     'version': specifications['version'],
+                                 })
+        validate_biosimulations_api_response(response, 'A Singularity image could not be generated for the Docker image')
+
     def post_entry_to_biosimulators_api(self, specifications, existing_version_specifications):
         """ Post the simulation to the BioSimulators database
 
@@ -377,26 +389,43 @@ class ValidateCommitSimulatorGitHubAction(GitHubAction):
             specifications (:obj:`dict`): specifications of a simulation tool
             existing_version_specifications (:obj:`list` of :obj:`dict`): specifications of other versions of simulation tool
         """
-        api_id = os.getenv('BIOSIMULATORS_API_CLIENT_ID')
-        api_secret = os.getenv('BIOSIMULATORS_API_CLIENT_SECRET')
-
-        response = requests.post(self.BIOSIMULATORS_AUTH_ENDPOINT, json={
-            'client_id': api_id,
-            'client_secret': api_secret,
-            'audience': self.BIOSIMULATORS_AUDIENCE,
-            "grant_type": "client_credentials",
-        })
-        response.raise_for_status()
-        response_data = response.json()
-        auth_headers = {'Authorization': response_data['token_type'] + ' ' + response_data['access_token']}
+        auth_headers = self.get_auth_headers_for_biosimulations_api(
+            self.config.biosimulators_auth_endpoint, self.config.biosimulators_audience,
+            self.config.biosimulators_api_client_id, self.config.biosimulators_api_client_secret)
 
         existing_versions = [existing_version_spec['version'] for existing_version_spec in existing_version_specifications]
         update_simulator = specifications['version'] in existing_versions
         if update_simulator:
-            endpoint = '{}simulators/{}/{}'.format(self.BIOSIMULATORS_API_ENDPOINT, specifications['id'], specifications['version'])
+            endpoint = '{}simulators/{}/{}'.format(self.config.biosimulators_api_endpoint, specifications['id'], specifications['version'])
             requests_method = requests.put
+            method = 'updated'
         else:
-            endpoint = '{}simulators'.format(self.BIOSIMULATORS_API_ENDPOINT)
+            endpoint = '{}simulators'.format(self.config.biosimulators_api_endpoint)
             requests_method = requests.post
+            method = 'added'
         response = requests_method(endpoint, headers=auth_headers, json=specifications)
+        validate_biosimulations_api_response(response, 'Simulation tool `{}` could not be {} to the BioSimulators registry.'.format(
+            specifications['id'], method))
+
+    def get_auth_headers_for_biosimulations_api(self, endpoint, audience, client_id, client_secret):
+        """ Get authorization headers for using one of the BioSimulations REST APIs
+        (BioSimulators, runBioSimulations, etc.).
+
+        Args:
+            endpoint (:obj:`str`): URL for getting an authentication token
+            audience (:obj:`str`): API audience
+            client_id (:obj:`str`): id for this client
+            client_secret (:obj:`str`): secret for this client
+
+        Returns:
+            :obj:`dict`: authorization headers
+        """
+        response = requests.post(endpoint, json={
+            'client_id': client_id,
+            'client_secret': client_secret,
+            'audience': audience,
+            "grant_type": "client_credentials",
+        })
         response.raise_for_status()
+        response_data = response.json()
+        return {'Authorization': response_data['token_type'] + ' ' + response_data['access_token']}
